#include "ChromeUtilsBinding.h"
#include "DOMErrorBinding.h"
#include "DominatorTreeBinding.h"
#include "ErrorEventBinding.h"
#include "EventBinding.h"
#include "EventTargetBinding.h"
#include "FileReaderBinding.h"
#include "HeapSnapshotBinding.h"
#include "MessageEventBinding.h"
#include "MessagePortBinding.h"
#include "PromiseBinding.h"
#include "PromiseDebuggingBinding.h"
#include "TCPServerSocketBinding.h"
#include "TCPServerSocketEventBinding.h"
#include "TCPSocketBinding.h"
#include "TCPSocketErrorEventBinding.h"
#include "TCPSocketEventBinding.h"
#include "TextDecoderBinding.h"
#include "TextEncoderBinding.h"
#include "ThreadSafeChromeUtilsBinding.h"
#include "URLSearchParamsBinding.h"
#include "WorkerBinding.h"
#include "js/Id.h"
#include "mozilla/dom/BindingUtils.h"
#include "nsThreadUtils.h"

namespace mozilla {
namespace dom {
bool
ResolveSystemBinding(JSContext* aCx, JS::Handle<JSObject*> aObj, JS::Handle<jsid> aId, bool* aResolvedp)
{
  MOZ_ASSERT(NS_IsMainThread());

  static jsid sChromeUtils_id;
  static jsid sChromeWorker_id;
  static jsid sDOMError_id;
  static jsid sDominatorTree_id;
  static jsid sErrorEvent_id;
  static jsid sEvent_id;
  static jsid sEventTarget_id;
  static jsid sFileReader_id;
  static jsid sHeapSnapshot_id;
  static jsid sMessageEvent_id;
  static jsid sMessagePort_id;
  static jsid sPromise_id;
  static jsid sPromiseDebugging_id;
  static jsid sTCPServerSocket_id;
  static jsid sTCPServerSocketEvent_id;
  static jsid sTCPSocket_id;
  static jsid sTCPSocketErrorEvent_id;
  static jsid sTCPSocketEvent_id;
  static jsid sTextDecoder_id;
  static jsid sTextEncoder_id;
  static jsid sThreadSafeChromeUtils_id;
  static jsid sURLSearchParams_id;
  static jsid sWorker_id;

  static bool idsInited = false;
  if (!idsInited) {
    if (!AtomizeAndPinJSString(aCx, sChromeUtils_id, "ChromeUtils")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sChromeWorker_id, "ChromeWorker")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sDOMError_id, "DOMError")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sDominatorTree_id, "DominatorTree")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sErrorEvent_id, "ErrorEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sEvent_id, "Event")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sEventTarget_id, "EventTarget")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sFileReader_id, "FileReader")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sHeapSnapshot_id, "HeapSnapshot")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sMessageEvent_id, "MessageEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sMessagePort_id, "MessagePort")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sPromise_id, "Promise")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sPromiseDebugging_id, "PromiseDebugging")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPServerSocket_id, "TCPServerSocket")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPServerSocketEvent_id, "TCPServerSocketEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPSocket_id, "TCPSocket")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPSocketErrorEvent_id, "TCPSocketErrorEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTCPSocketEvent_id, "TCPSocketEvent")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTextDecoder_id, "TextDecoder")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sTextEncoder_id, "TextEncoder")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sThreadSafeChromeUtils_id, "ThreadSafeChromeUtils")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sURLSearchParams_id, "URLSearchParams")) {
      return false;
    }

    if (!AtomizeAndPinJSString(aCx, sWorker_id, "Worker")) {
      return false;
    }

    idsInited = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sChromeUtils_id) && ChromeUtilsBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ChromeUtilsBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sChromeWorker_id) && ChromeWorkerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ChromeWorkerBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sDOMError_id) {
    if (!DOMErrorBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sDominatorTree_id) && DominatorTreeBinding::ConstructorEnabled(aCx, aObj)) {
    if (!DominatorTreeBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sErrorEvent_id) {
    if (!ErrorEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sEvent_id) {
    if (!EventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sEventTarget_id) {
    if (!EventTargetBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sFileReader_id) {
    if (!FileReaderBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sHeapSnapshot_id) && HeapSnapshotBinding::ConstructorEnabled(aCx, aObj)) {
    if (!HeapSnapshotBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sMessageEvent_id) {
    if (!MessageEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sMessagePort_id) {
    if (!MessagePortBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sPromise_id) {
    if (!PromiseBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sPromiseDebugging_id) && PromiseDebuggingBinding::ConstructorEnabled(aCx, aObj)) {
    if (!PromiseDebuggingBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPServerSocket_id) && TCPServerSocketBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPServerSocketBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPServerSocketEvent_id) && TCPServerSocketEventBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPServerSocketEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPSocket_id) && TCPSocketBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPSocketBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPSocketErrorEvent_id) && TCPSocketErrorEventBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPSocketErrorEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sTCPSocketEvent_id) && TCPSocketEventBinding::ConstructorEnabled(aCx, aObj)) {
    if (!TCPSocketEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sTextDecoder_id) {
    if (!TextDecoderBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sTextEncoder_id) {
    if (!TextEncoderBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sThreadSafeChromeUtils_id) && ThreadSafeChromeUtilsBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ThreadSafeChromeUtilsBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sURLSearchParams_id) {
    if (!URLSearchParamsBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sWorker_id) && WorkerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!WorkerBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  return true;
}

} // namespace dom
} // namespace mozilla

